@@ -6,11 +6,11 @@ from .. import util
 
 class MultipleNegativesRankingLoss(nn.Module):
     """
-        This loss expects as input a batch consisting of sentence pairs (a_1, b_1), (a_2, b_2)..., (a_n, b_n)
-        where we assume that (a_i, b_i) are a positive pair and (a_i, b_j) for i!=j a negative pair.
+        This loss expects as input a batch consisting of sentence pairs (a_1, p_1), (a_2, p_2)..., (a_n, p_n)
+        where we assume that (a_i, p_i) are a positive pair and (a_i, p_j) for i!=j a negative pair.
 
-        For each a_i, it uses all other b_j as negative samples, i.e., for a_i, we have 1 positive example (b_i) and
-        n-1 negative examples (b_j). It then minimizes the negative log-likehood for softmax normalized scores.
+        For each a_i, it uses all other p_j as negative samples, i.e., for a_i, we have 1 positive example (p_i) and
+        n-1 negative examples (p_j). It then minimizes the negative log-likehood for softmax normalized scores.
 
         This loss function works great to train embeddings for retrieval setups where you have positive pairs (e.g. (query, relevant_doc))
         as it will sample in each batch n-1 negative docs randomly.
@@ -20,12 +20,10 @@ class MultipleNegativesRankingLoss(nn.Module):
         For more information, see: https://arxiv.org/pdf/1705.00652.pdf
         (Efficient Natural Language Response Suggestion for Smart Reply, Section 4.4)
 
-        The error function is equivalent to::
+        You can also provide one or multiple hard negatives per anchor-positive pair by structering the data like this:
+        (a_1, p_1, n_1), (a_2, p_2, n_2)
 
-            scores = self.similarity_fct(embeddings_a, embeddings_b) * self.scale
-            labels = torch.tensor(range(len(scores)), dtype=torch.long).to(self.model.device) #Example a[i] should match with b[i]
-            cross_entropy_loss = nn.CrossEntropyLoss()
-            return cross_entropy_loss(scores, labels)
+        Here, n_1 is a hard negative for (a_1, p_1). The loss will use for the pair (a_i, p_i) all p_j (j!=i) and all n_j as negatives.
 
         Example::
 
@@ -39,7 +37,7 @@ class MultipleNegativesRankingLoss(nn.Module):
             train_dataloader = DataLoader(train_dataset, shuffle=True, batch_size=train_batch_size)
             train_loss = losses.MultipleNegativesRankingLoss(model=model)
     """
-    def __init__(self, model: SentenceTransformer, scale: float = 20.0, similarity_fct = util.pytorch_cos_sim):
+    def __init__(self, model: SentenceTransformer, scale: float = 20.0, similarity_fct = util.cos_sim):
         """
         :param model: SentenceTransformer model
         :param scale: Output of similarity function is multiplied by scale value
@@ -49,27 +47,19 @@ class MultipleNegativesRankingLoss(nn.Module):
         self.model = model
         self.scale = scale
         self.similarity_fct = similarity_fct
+        self.cross_entropy_loss = nn.CrossEntropyLoss()
 
 
     def forward(self, sentence_features: Iterable[Dict[str, Tensor]], labels: Tensor):
         reps = [self.model(sentence_feature)['sentence_embedding'] for sentence_feature in sentence_features]
-        reps_a, reps_b = reps
-        return self.multiple_negatives_ranking_loss(reps_a, reps_b)
+        embeddings_a = reps[0]
+        embeddings_b = torch.cat(reps[1:])
 
-
-    def multiple_negatives_ranking_loss(self, embeddings_a: Tensor, embeddings_b: Tensor):
-        """
-        :param embeddings_a:
-            Tensor of shape (batch_size, embedding_dim)
-        :param embeddings_b:
-            Tensor of shape (batch_size, embedding_dim)
-        :return:
-            The scalar loss
-        """
         scores = self.similarity_fct(embeddings_a, embeddings_b) * self.scale
-        diagonal_mean = torch.mean(torch.diag(scores))
-        mean_log_row_sum_exp = torch.mean(torch.logsumexp(scores, dim=1))
-        return -diagonal_mean + mean_log_row_sum_exp
+        labels = torch.tensor(range(len(scores)), dtype=torch.long, device=scores.device)  # Example a[i] should match with b[i]
+        return self.cross_entropy_loss(scores, labels)
+
+
 
 
 
